package com.gic.evaluate.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.OrderReplyEnum;
import com.gic.evaluate.constant.ReplyEnum;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.EvaluateReplyDTO;
import com.gic.evaluate.dto.MemberReplyDTO;
import com.gic.evaluate.entity.*;
import com.gic.evaluate.qo.EvaluateOutQO;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.service.*;
import com.gic.member.api.dto.AcuDetailDTO;
import com.gic.member.api.dto.MemberUserDTO;
import com.gic.member.api.service.CuDetailService;
import com.gic.member.api.service.MemberUserApiService;
import com.gic.order.api.dto.OrderListDetailDTO;
import com.gic.order.api.service.OrderApiService;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 15:27
 */
@Service("evaluateOutApiService")
public class EvaluateOutApiServiceImpl implements EvaluateOutApiService {

    private Logger logger = LogManager.getLogger(EvaluateOutApiServiceImpl.class);
    
    @Autowired
    private EvaluateService evaluateService;
    @Autowired
    private EvaluateReplyService evaluateReplyService;
    @Autowired
    private EvaluateOrderConfigService evaluateOrderConfigService;
    @Autowired
    private BadEvaluateConfigService badEvaluateConfigService;
    @Autowired
    private EvaluateImgService evaluateImgService;
    @Autowired
    private CuDetailService cuDetailService;
    @Autowired
    private MemberUserApiService memberUserApiService;
    @Autowired
    private OrderApiService orderApiService;
    @Autowired
    private StoreApiService storeApiService;

    @Override
    public ServiceResponse<Void> saveEvaluate(EvaluateDTO evaluateDTO) {
        logger.info("保存订单评价:{}", JSON.toJSONString(evaluateDTO));
        TabEvaluate evaluate = evaluateService.getEvaluateByOrderId(evaluateDTO.getEnterpriseId(), evaluateDTO.getOrderId());
        if (evaluate != null) {
            EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS);
        }
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(evaluateDTO.getEnterpriseId());
        OrderListDetailDTO orderListDetailDTO = orderApiService.getOrderDetail(evaluateDTO.getEnterpriseId(), evaluateDTO.getOrderId(), evaluateDTO.getMemberId()).getResult();

        // 判断是否超过评价时间
        if (DateUtil.compareDate(new Date(), DateUtil.addDay(orderListDetailDTO.getOrderDTO().getReceiptsTime(), evaluateOrderConfig.getReviewLimitday()))) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已超越评价期限，无法评价");
        }

        evaluateDTO.setGoodsInfo(JSON.toJSONString(orderListDetailDTO.getOrderItemList()));
        StoreDTO storeDTO = storeApiService.getStoreById(evaluateDTO.getEnterpriseId(), Integer.valueOf(orderListDetailDTO.getOrderDTO().getStoreIdOffline())).getResult();
        evaluateDTO.setStoreName(storeDTO.getStoreName());
        evaluateDTO.setStoreCode(storeDTO.getStoreCode());
        evaluateDTO.setStoreGroupId(storeDTO.getStoreGroupId());
        evaluateDTO.setStoreGroupName(storeDTO.getStoreGroupName());
        evaluateDTO.setAreaName(storeDTO.getProvinces());
        evaluateDTO.setOrderNumber(orderListDetailDTO.getOrderDTO().getOrderNo());
        evaluateDTO.setReceiptsDate(orderListDetailDTO.getOrderDTO().getReceiptsTime());
        evaluateDTO.setOrderMount(orderListDetailDTO.getOrderDTO().getPayAmount());
        evaluateDTO.setClerkId(orderListDetailDTO.getOrderDTO().getClerkId());
        evaluateDTO.setClerkName(orderListDetailDTO.getOrderDTO().getClerkName());

        evaluateDTO.setReviewStatus(Constants.CLOSE);
        evaluateDTO.setTopHotStatus(Constants.CLOSE);
        evaluateDTO.setMaliciousStatus(Constants.CLOSE);
        evaluateDTO.setShowStatus(Constants.OPEN);
        evaluateDTO.setVisitStatus(0);
        evaluateDTO.setReplyStatus(ReplyEnum.WAIT.getCode());
        TabBadEvaluateConfig badEvaluateConfig = badEvaluateConfigService.getBadEvaluateConfig(evaluateDTO.getEnterpriseId());
        Integer badEvaluate = isBadEvaluate(badEvaluateConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        Integer goodEvaluate = isGoodEvaluate(badEvaluateConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        evaluateDTO.setBadEvaluateStatus(badEvaluate);
        evaluateDTO.setGoodEvaluateStatus(goodEvaluate);


        if (evaluateDTO.getAcuId() != null) {
            ServiceResponse<AcuDetailDTO> acuDetailResponse = cuDetailService.getAcuDetail(evaluateDTO.getAcuId(), evaluateDTO.getEnterpriseId(), evaluateDTO.getEcuId());
            if (acuDetailResponse.isSuccess() || acuDetailResponse.getResult() != null) {
                evaluateDTO.setMemberImgUrl(acuDetailResponse.getResult().getHeadUrl());
            }
        }

        Integer evaluateId = evaluateService.saveEvaluate(evaluateDTO);

        if (CollectionUtils.isNotEmpty(evaluateDTO.getImgList())) {
            evaluateImgService.saveList(evaluateDTO.getEnterpriseId(), evaluateId, evaluateDTO.getImgList());
        }

        return EnterpriseServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveReply(MemberReplyDTO memberReplyDTO) {
        logger.info("追评订单评价:{}", JSON.toJSONString(memberReplyDTO));
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(memberReplyDTO.getEnterpriseId());

        // 判断是否支持
        if (Constants.CLOSE.equals(evaluateOrderConfig.getReview())) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不支持追评");
        }
        // 是否超过期限
        TabEvaluate evaluate = this.evaluateService.getEvaluate(memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEvaluateId());
        if (DateUtil.compareDate(new Date(), DateUtil.addDay(evaluate.getReceiptsDate(), evaluateOrderConfig.getReviewLimitday()))) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已超越评价期限，无法评价");
        }
        // 是否超过次数
        List<TabEvaluateReply> replyList = this.evaluateReplyService.listEvaluateReply(memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEvaluateId());
        if (CollectionUtils.isNotEmpty(replyList)) {
            if (evaluateOrderConfig.getReviewTimes() <= replyList.size()) {
                return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经超过最大追评次数，无法继续追评");
            }
        }

        ServiceResponse<MemberUserDTO> memberUserDTOServiceResponse = memberUserApiService.getByMcuId(memberReplyDTO.getMemberId(), memberReplyDTO.getEnterpriseId());
        if (memberUserDTOServiceResponse.isSuccess()) {
            memberReplyDTO.setMemberName(memberUserDTOServiceResponse.getResult().getName());
            ServiceResponse<AcuDetailDTO> acuDetailResponse = cuDetailService.getAcuDetail(memberReplyDTO.getAcuId(), memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEcuId());
            if (acuDetailResponse.isSuccess() || acuDetailResponse.getResult() != null) {
                memberReplyDTO.setMemberImgUrl(acuDetailResponse.getResult().getHeadUrl());
            }

        }

        // 保存追评
        Integer evaluateRelayId = evaluateReplyService.saveMemberReplyEvaluate(memberReplyDTO);

        // 更新追评状态
        EvaluateDTO evaluateDTO = new EvaluateDTO();
        evaluateDTO.setEvaluateId(memberReplyDTO.getEvaluateId());
        evaluateDTO.setReplyStatus(ReplyEnum.HAS_NEW.getCode());
        evaluateService.updateEvaluate(evaluateDTO);

        // 保存追评图片
        //        void saveList(Integer enterpriseId, Integer evaluateId, Integer evaluateReplyId, List<String> imgUrlList);
        evaluateImgService.saveList(memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEvaluateId(), evaluateRelayId, memberReplyDTO.getImgList());

        return EnterpriseServiceResponse.success();
    }

/*
    @Override
    public ServiceResponse<Page<EvaluateDTO>> listEvaluateByStore(EvaluateOutQO evaluateOutQO) {
        EvaluateQO evaluateQO = new EvaluateQO();
        evaluateQO.setCurrentPage(evaluateOutQO.getCurrentPage());
        evaluateQO.setPageSize(evaluateOutQO.getPageSize());
        evaluateQO.setEnterpriseId(evaluateOutQO.getEnterpriseId());
        evaluateQO.setStoreIdList(Collections.singletonList(evaluateOutQO.getStoreId()));
//        evaluateQO.setMemberId(evaluateOutQO.getMemberId());

        com.github.pagehelper.Page<TabEvaluate> evaluateList = evaluateService.listEvaluate(evaluateQO);
        // 看到图在写
        return null;
    }
*/

    @Override
    public ServiceResponse<List<EvaluateDTO>> listEvaluateByMember(EvaluateOutQO evaluateOutQO) {
        EvaluateQO evaluateQO = new EvaluateQO();
        evaluateQO.setEnterpriseId(evaluateOutQO.getEnterpriseId());
        evaluateQO.setMemberId(evaluateOutQO.getMemberId());
        evaluateQO.setShowStatus(Constants.OPEN);
        List<TabEvaluate> evaluateList = this.evaluateService.listEvaluate(evaluateQO);
        if (CollectionUtils.isEmpty(evaluateList)) {
            return EnterpriseServiceResponse.success(Collections.emptyList());
        }
        List<EvaluateDTO> evaluateDTOList = EntityUtil.changeEntityListByJSON(EvaluateDTO.class, evaluateList);
//        List<Integer> evaluateIdList = evaluateDTOList.stream().map(EvaluateDTO::getEvaluateId).collect(Collectors.toList());
        // 有空去掉循环
        for (EvaluateDTO evaluateDTO : evaluateDTOList) {
            // 图片
            List<TabEvaluateImg> evaluateImgList = evaluateImgService.listImageByEvaluateId(evaluateDTO.getEvaluateId());
            List<String> imgList = Optional.ofNullable(evaluateImgList).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
            evaluateDTO.setImgList(imgList);
            // 追评和回复
            List<TabEvaluateReply> evaluateReplyList = this.evaluateReplyService.listEvaluateReply(evaluateDTO.getEnterpriseId(), evaluateDTO.getEvaluateId());
            List<EvaluateReplyDTO> replyDTOList = EntityUtil.changeEntityListByJSON(EvaluateReplyDTO.class, evaluateReplyList);
            // 追评和回复的图片
            for (EvaluateReplyDTO replyDTO : replyDTOList) {
                List<TabEvaluateImg> evaluateReplyImgList = evaluateImgService.listImageByEvaluateReplyId(replyDTO.getEvaluateReplyId());
                List<String> replyImgList = Optional.ofNullable(evaluateReplyImgList).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
                replyDTO.setImgList(replyImgList);
            }
            evaluateDTO.setReplyList(replyDTOList);
        }
        return EnterpriseServiceResponse.success(evaluateDTOList);
    }

    @Override
    public ServiceResponse<Map<Long, Integer>> queryRelayStatusByOrderId(Integer enterpriseId, List<Long> orderIdList) {
        Map<Long, Integer> resultMap = new HashMap<>();
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(enterpriseId);


        for (Long orderId : orderIdList) {
            TabEvaluate evaluate = this.evaluateService.getEvaluateByOrderId(enterpriseId, orderId);
            if (evaluate != null) {
                // 已经评价过
                if (Constants.CLOSE.equals(evaluateOrderConfig.getReview())) {
                    // 判断追评是否支持
                    resultMap.put(orderId, OrderReplyEnum.REPLY_NOT_SUPPORT.getCode());
                } else if (DateUtil.compareDate(new Date(), DateUtil.addDay(evaluate.getReceiptsDate(), evaluateOrderConfig.getReviewLimitday()))) {
                    // 追评超过期限
                    resultMap.put(orderId, OrderReplyEnum.REPLY_OVERDUE.getCode());
                }else {
                    // 是否追评超过次数
                    List<TabEvaluateReply> replyList = this.evaluateReplyService.listEvaluateReply(evaluate.getEnterpriseId(), evaluate.getEvaluateId());
                    if (CollectionUtils.isNotEmpty(replyList) && evaluateOrderConfig.getReviewTimes() <= replyList.size()) {
                        // 追评超过次数
                        resultMap.put(orderId, OrderReplyEnum.REPLY_OVER_MAX_TIMES.getCode());
                    } else {
                        //追评没有超过次数
                        resultMap.put(orderId, evaluate.getReplyStatus());
                    }
                }
            } else {
                //还没有评价过
                // todo 是否超过评价时间
                resultMap.put(orderId, -1);
            }
        }

        return EnterpriseServiceResponse.success(resultMap);
    }

    private Integer isBadEvaluate(TabBadEvaluateConfig badEvaluateConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        // 任何一项评价达到设定值
        if (Constants.OPEN.equals(badEvaluateConfig.getBadOnceStatus())) {
            if (matchConfigValue(badEvaluateConfig.getBadGoodsType(), badEvaluateConfig.getBadGoodsScore(), goodsScore)) {
                // 商品质量
                return Constants.OPEN;
            }
            if (matchConfigValue(badEvaluateConfig.getBadStoreType(), badEvaluateConfig.getBadStoreScore(), storeScore)) {
                // 门店服务
                return Constants.OPEN;
            }
            if (matchConfigValue(badEvaluateConfig.getBadServiceType(), badEvaluateConfig.getBadServiceScore(), serviceScore)) {
                // 服务能力
                return Constants.OPEN;
            }
        }
        // 三项评价均分
        if (Constants.OPEN.equals(badEvaluateConfig.getBadTotalStatus())) {
            if (matchConfigValue(badEvaluateConfig.getBadStoreType(), badEvaluateConfig.getBadStoreScore() * 3, goodsScore + storeScore + serviceScore)) {
                return Constants.OPEN;
            }
        }
        return Constants.CLOSE;
    }

    private Integer isGoodEvaluate(TabBadEvaluateConfig badEvaluateConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        // 任何一项评价达到设定值
        if (Constants.OPEN.equals(badEvaluateConfig.getGoodStatus())) {
            // 三项评价均分
            if (matchConfigValue(badEvaluateConfig.getGoodTotalType(), badEvaluateConfig.getGoodTotalScore() * 3, goodsScore + storeScore + serviceScore)) {
                if (matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), goodsScore)) {
                    // 商品质量
                    return Constants.OPEN;
                }
                if (matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), storeScore)) {
                    // 门店服务
                    return Constants.OPEN;
                }
                if (matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), serviceScore)) {
                    // 服务能力
                    return Constants.OPEN;
                }
            }
        }
        return Constants.CLOSE;
    }

    private boolean matchConfigValue(Integer computeType, Integer computeScore, Integer score) {
        switch (computeType) {
            case 1:
                return score < computeScore;
            case 2:
                return score <= computeScore;
            case 3:
                return score.equals(computeScore);
            case 4:
                return score >= computeScore;
            case 5:
                return score > computeScore;
            default:
                break;
        }
        return false;
    }
}
